/*
 * Copyright (c) 1995-2011, Index Data
 * All rights reserved.
 * See the file LICENSE for details.
 */

package com.indexdata.torus2.model;

import com.indexdata.torus2.model.Facets.Facet.FacetTerm;
import com.indexdata.torus2.query.QueryException;
import com.indexdata.utils.XmlUtils;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 *
 * @author jakub
 */
public class Facets {
  private List<Facet> facets = new ArrayList<Facet>();

  public Facets(List<String> facetNames) throws QueryException {
    for (String fname : facetNames) {
      if (!fname.isEmpty()) {
        String[] pair = fname.split(":");
        switch (pair.length) {
          case 1: facets.add(new Facet(pair[0], null)); break;
          case 2:
            try {
              facets.add(new Facet(pair[0], Pattern.compile(pair[1])));
            } catch (PatternSyntaxException pse) {
              throw new QueryException("Cannot parse facet split regex at '"+fname+"'", pse);
            }
            break;
          //reuse query exception for this
          default: throw new QueryException("Cannot parse facet name or split regex at'"+fname+"'");
        }
      }
    }
  }

  public List<Facet> getFacets() {
    return facets;
  }

  public Facet getFacet(String name) {
    for (Facet f : facets) {
      if (f.name.equals(name)) return f;
    }
    return null;
  }

  public Document toDom() {
    Document fDoc = XmlUtils.newDoc("facets");
    for (Facet f : getFacets()) {
      Element fNode = fDoc.createElement("facet");
      fNode.setAttribute("name", f.getName());
      for (FacetTerm t : f.getTerms()) {
        Element tNode = fDoc.createElement("term");
        tNode.setAttribute("content", t.getTerm());
        tNode.setAttribute("count", Integer.toString(t.getCount()));
        fNode.appendChild(tNode);
      }
      fDoc.getDocumentElement().appendChild(fNode);
    }
    return fDoc;
  }

  public class Facet {
    private String name;
    private Pattern splitRegex;
    private List<FacetTerm> terms = new ArrayList<FacetTerm>();

    public class FacetTerm {
      private String term;
      private int count;

      private FacetTerm(String term, int count) {
        this.term = term;
        this.count = count;
      }

      public int getCount() {
        return count;
      }

      public String getTerm() {
        return term;
      }

    }

    public Facet(String name, Pattern splitRegex) {
      this.name = name;
      this.splitRegex = splitRegex;
    }

    public String getName() {
      return name;
    }

    public List<FacetTerm> getTerms() {
      return terms;
    }

    public Pattern getSplitRegex() {
      return splitRegex;
    }

    public FacetTerm addTerm(String term) {
      for (FacetTerm ft : terms) {
        if (ft.term.equals(term)) {
          ft.count++;
          return ft;
        }
      }
      FacetTerm ft = new FacetTerm(term, 1);
      if (terms.add(ft))
        return ft;
      else
        throw new ArrayStoreException("storing term failed");
    }
  }
}
