/*
 * Copyright (c) 1995-2012, Index Datassss
 * All rights reserved.
 * See the file LICENSE for details.
 */
package com.indexdata.torus2.query;

import com.indexdata.torus2.query.Tokenizer;
import com.indexdata.torus2.query.QueryException;
import java.io.IOException;
import org.z3950.zing.cql.CQLNode;
import org.z3950.zing.cql.CQLParseException;
import org.z3950.zing.cql.CQLParser;

/**
 *
 * @author jakub
 */
public class MatchingUtil {
  /**
   * Parses query string into tree representation
   *
   * @param query
   * @return parsed query or null if query string was null or empty
   * @throws QueryException when query is malformed or cannot be read
   */
  public static CQLNode parseQuery(String query) throws QueryException {
    try {
      if (query == null || query.isEmpty())
        return null;
      else {
        CQLParser parser = new CQLParser(CQLParser.V1POINT2);
        //the following relations will be mapped to within/isoDate
        parser.registerCustomRelation("@");
        parser.registerCustomRelation("at");
        return parser.parse(query);
      }
    } catch (CQLParseException pe) {
      throw new QueryException("Parsing query failed - "
        + pe.getMessage() + " at character " + pe.getPosition(), pe);
    } catch (IOException io) {
      throw new QueryException("Parsing query failed - input error", io);
    }
  }

  public static boolean equalsWildcard(String pattern, int patternAt,
    String text, int textAt) {
    int i = patternAt;
    int j = textAt;
    boolean escaped = false;
    for (; i != pattern.length() && j != text.length(); i++, j++) {
      char p = pattern.charAt(i);
      char t = text.charAt(j);
      if (escaped) {
        escaped = false;
      } else {
        if (p == '\\') {
          escaped = true;
          //the text should not move
          --j;
          continue;
        }
        if (p == '*') {
          //skip consecutive stars
          while ((i < (pattern.length() - 1)) && pattern.charAt(i + 1) == '*')
            i++;
          // trailing star matches rest
          if (i == pattern.length() - 1)
            return true;
          //non-greedy, should we backtrack instead?
          for (int k = j; k < text.length(); k++) {
            if (equalsWildcard(pattern, i + 1, text, k)) {
              return true;
            }
          }
          continue;
        }
        if (p == '?')
          continue;
      }
      if (p != t)
        return false;
    }
    //make sure stars run out
    while ((i < (pattern.length())) && pattern.charAt(i) == '*')
      i++;
    // text and patern must run out
    return i == pattern.length() && j == text.length();
  }

  public static String normalizeTokens(String src, String sep,
    Tokenizer.TokenizerType type) {
    Tokenizer t = new Tokenizer(src, type);
    StringBuilder sb = new StringBuilder(src.length());
    String s = "";
    while (t.hasMoreElements()) {
      sb.append(s);
      sb.append(t.nextElement());
      s = sep;
    }
    return sb.toString();
  }

  public static String deAnchorTerm(String term) {
    if (term.isEmpty()) return "*  *";
    term = term.charAt(0) == '^' ? " " + term.substring(1) : "* " + term;
    term = term.charAt(term.length() - 1) == '^'
      ? term.substring(0, term.length() - 1) + " " : term + " *";
    return term;
  }

  public static String surround(String text, String sur) {
    return sur + text + sur;
  }
}
