/*
 * Copyright (c) 1995-2012, Index Datassss
 * All rights reserved.
 * See the file LICENSE for details.
 */
package com.indexdata.torus2.query;

import com.indexdata.torus2.util.EscapingStringTokenizer;

import java.util.Enumeration;
import java.util.StringTokenizer;

/**
 *
 * @author jakub
 */
public class Tokenizer implements Enumeration<String> {
    private final static String TOKEN_DELIMS  = " \t\n\r\f.,!;:'\"_+-=<>/|&()@~`";
    private final static String SPECIAL_CHARS = "\\?*^";
    // field token delims MUST include CQL masking chars because the escape chars
    // are stripped from the tokens
    public final static String FIELD_TOKEN_DELIMS = TOKEN_DELIMS + SPECIAL_CHARS;
    public final static String QUERY_TOKEN_DELIMS = TOKEN_DELIMS;
    
    @SuppressWarnings("rawtypes")
    private Enumeration tokenizer;
    
    public Tokenizer(String str, TokenizerType type) {
      switch(type) {
        case FIELD: tokenizer = 
          new StringTokenizer(str, FIELD_TOKEN_DELIMS, false); break;
        case QUERY: tokenizer = 
          new EscapingStringTokenizer(str, QUERY_TOKEN_DELIMS, "", '\\', false, 
            FIELD_TOKEN_DELIMS); break;
        default: throw new IllegalArgumentException("Uknown tokenizer type "+type);
      }
    } 

    @Override
    public boolean hasMoreElements() {
      return tokenizer.hasMoreElements();
    }

    @Override
    public String nextElement() {
      return ((String) tokenizer.nextElement()).toLowerCase();
    }
    
    public enum TokenizerType {
      QUERY, FIELD
    }
}
