/*
 * Copyright (c) 1995-2012, Index Datassss
 * All rights reserved.
 * See the file LICENSE for details.
 */
package com.indexdata.torus2.query.match;

import com.indexdata.torus2.query.match.term.NotEqualTermMatcher;
import com.indexdata.torus2.query.match.term.IpWithinMatcher;
import com.indexdata.torus2.query.match.term.DateWithinMatcher;
import com.indexdata.torus2.query.match.term.IpEnclosesMatcher;
import com.indexdata.torus2.query.match.term.ExactTermMatcher;
import com.indexdata.torus2.query.match.term.AllTermMatcher;
import com.indexdata.torus2.query.match.term.AdjacencyTermMatcher;
import com.indexdata.torus2.query.match.term.AnyTermMatcher;
import com.indexdata.torus2.query.QueryException;
import org.apache.log4j.Logger;
import org.z3950.zing.cql.CQLTermNode;
import org.z3950.zing.cql.Modifier;

/**
 *
 * @author jakub
 */
public class TypeAwareTermMatcher implements TermMatcher {
  //term-specific matcher
  private TermMatcher matcher;
  private static Logger logger = Logger.getLogger("com.indexdata.torus2");
  //keep for logging
  private String index;
  private String relation;
  private String term;

  public TypeAwareTermMatcher(CQLTermNode tn, String index) throws
    QueryException {
    matcher = getMatcher(tn, index);
  }
  
  private TermMatcher getMatcher(CQLTermNode tn, String index) throws QueryException {
    this.index = index;
    this.relation = tn.getRelation().getBase();
    this.term = tn.getTerm();
    if (relation.equals("==") || relation.equals("exact")) {
      return new ExactTermMatcher(term);
    } else if (relation.equals("<>")) {
      return new NotEqualTermMatcher(term);
    } else if (relation.equals("adj") || relation.equals("=")
      || relation.equals("scr")) {
      return new AdjacencyTermMatcher(term);
    } else if (relation.equals("any")) {
      return new AnyTermMatcher(term);
    } else if (relation.equals("all")) {
      return new AllTermMatcher(term);
    } else if (relation.equals("within")) {
      for (Modifier m : tn.getRelation().getModifiers()) {
        String type = m.getType();
        if (type.equals("net.ipaddress")) {
          return new IpWithinMatcher(term);
        } else if (type.equalsIgnoreCase("rfcDate")
          || type.equalsIgnoreCase("isoDate")) {
          return new DateWithinMatcher(term, type);
        } else {
          throw new QueryException("CQL error: unsupported modifier '"
            + m.getType() + "'");
        }
      }
      throw new QueryException("CQL error: relation '"
        + relation + "' expects a modifier");
    } else if (relation.equals("encloses")) {
      for (Modifier m : tn.getRelation().getModifiers()) {
        String type = m.getType();
        if (type.equals("net.ipaddress")) {
          return new IpEnclosesMatcher(term);
        } else {
          throw new QueryException("CQL error: unsupported modifier '"
            + m.getType() + "'");
        }
      }
      throw new QueryException("CQL error: relation '"
        + relation + "' expects a modifier");
    } else if (relation.equals("@") || relation.equals("at")) {
      return new DateWithinMatcher(term, "isoDate");
    } else {
      throw new QueryException("CQL error: unsupported relation '"
        + relation + "'");
    }
  }

  @Override
  public boolean match(String value) {
    //we only protect against null/empty here, rest is left to the actual matcher
    if (value == null)
      return false;
    if (logger.isDebugEnabled()) {
      logger.debug("Matching: '" + index + "' " + relation + " '" + term
        + "' on '" + value + "'");
    }
    return matcher.match(value);
  }
}
