/*
 * Copyright (c) 1995-2011, Index Data
 * All rights reserved.
 * See the file LICENSE for details.
 */

package com.indexdata.torus2.rw;

import com.indexdata.masterkey.config.MasterkeyConfiguration;
import com.indexdata.masterkey.config.MissingMandatoryParameterException;
import com.indexdata.masterkey.config.ModuleConfiguration;
import com.indexdata.torus2.rw.impl.CachingReaderWriter;
import com.indexdata.torus2.rw.impl.RealmFileRW;
import com.indexdata.torus2.exception.InternalIOException;
import java.io.IOException;
//import org.apache.log4j.Logger;

/**
 * Instantiates suitable realm reader/writer given configuration parameters.
 * @author jakub
 */
public class RealmRWFactory {
  //private static Logger logger = Logger.getLogger("com.indexdata.torus2");

  public static RealmReaderWriter getRealmRW(String realmName,
    MasterkeyConfiguration cfg, String host, String proto) throws InternalIOException {
    try {
      ModuleConfiguration storeCfg = cfg.getModuleConfiguration("storage");
      ModuleConfiguration cacheCfg = cfg.getModuleConfiguration("cache");
        return new CachingReaderWriter(new RealmFileRW(realmName,
          storeCfg.getMandatory("ROOT_PATH")), host, proto, 
          cacheCfg.get("HONOR_PROTOCOL", "no").equals("yes"),
          cacheCfg.get("HONOR_AUTHORITY", "no").equals("yes"),
          cacheCfg.get("HONOR_PORT", "no").equals("yes"));
    } catch (MissingMandatoryParameterException mmpe) {
      throw new InternalIOException(mmpe);
    } catch (IOException io) {
      throw new RuntimeException(io);
    }
  }

}
