/*
 * Copyright (c) 1995-2011, Index Data
 * All rights reserved.
 * See the file LICENSE for details.
 */

package com.indexdata.torus2.rw.impl;

import com.indexdata.torus2.exception.MalformedInputException;
import com.indexdata.torus2.model.ParentRealm;
import com.indexdata.torus2.model.RealmDefinition;
import com.indexdata.torus2.exception.InternalIOException;
import com.indexdata.torus2.model.Record.Layer;
import com.indexdata.torus2.rw.RealmReaderWriter;
import com.indexdata.torus2.util.Pair;
import java.util.Date;
import java.util.List;

/**
 *
 * @author jakub
 */
public class ProxyReaderWriter implements RealmReaderWriter {
  protected RealmReaderWriter rw;

  public ProxyReaderWriter(RealmReaderWriter rw) {
    this.rw = rw;
  }

  @Override
  public boolean create(RealmDefinition rd) throws InternalIOException {
    return rw.create(rd);
  }

  @Override
  public boolean open() throws InternalIOException {
    return rw.open();
  }
  
  @Override
  public boolean update(RealmDefinition rd) throws InternalIOException {
    return rw.update(rd);
  }
  
  @Override
  public RealmDefinition describe() {
    return rw.describe();
  }

  @Override
  public void destroy() throws InternalIOException {
    rw.destroy();
  }

  @Override
  public List<Layer> readOverrideRecords() throws InternalIOException {
    return rw.readOverrideRecords();
  }

  @Override
  public Layer readOverrideRecord(String id) throws InternalIOException {
    return rw.readOverrideRecord(id);
  }

  @Override
  public Layer writeOverrideRecord(String id, Layer doc) throws InternalIOException {
    return rw.writeOverrideRecord(id, doc);
  }

  @Override
  public Layer writeOverrideRecord(Layer doc) throws InternalIOException {
    return rw.writeOverrideRecord(doc);
  }

  @Override
  public void deleteOverrideRecord(String id) throws InternalIOException {
    rw.deleteOverrideRecord(id);
  }

  @Override
  public Pair<List<Layer>,Date> readWorldRecordsForParent(String prId) throws InternalIOException {
    return rw.readWorldRecordsForParent(prId);
  }

  @Override
  public Layer readWorldRecord(String id) throws InternalIOException, MalformedInputException {
    return rw.readWorldRecord(id);
  }

  @Override
  public List<Layer> writeWorldRecordsForParent(List<Layer> recs, String prId) throws InternalIOException {
    return rw.writeWorldRecordsForParent(recs, prId);
  }

  @Override
  public void deleteWorldRecordsByParentId(String prId) throws InternalIOException {
    rw.deleteWorldRecordsByParentId(prId);
  }

  @Override
  public ParentRealm readParentRealm(String prId) throws InternalIOException {
    return rw.readParentRealm(prId);
  }

  @Override
  public ParentRealm writeParentRealm(String prId, ParentRealm pr) throws InternalIOException {
    return rw.writeParentRealm(prId, pr);
  }

  @Override
  public ParentRealm writeParentRealm(ParentRealm pr) throws InternalIOException {
    return rw.writeParentRealm(pr);
  }

  @Override
  public void deleteParentRealm(String prId) throws InternalIOException {
    rw.deleteParentRealm(prId);
  }

  @Override
  public List<ParentRealm> readParentRealms() throws InternalIOException {
    return rw.readParentRealms();
  }

  @Override
  public List<RealmDefinition> listRealms() throws InternalIOException {
    return rw.listRealms();
  }

  @Override
  public boolean isCachedAsParent() throws InternalIOException {
    return rw.isCachedAsParent();
  }

  @Override
  public Layer updateCachedAsOriginal(Layer original) throws InternalIOException {
    return rw.updateCachedAsOriginal(original);
  }

  @Override
  public void deleteCachedAsOriginal(String id) throws InternalIOException {
    rw.deleteCachedAsOriginal(id);
  }

  @Override
  public Layer readMasterRecord(String id) throws InternalIOException {
    return rw.readMasterRecord(id);
  }

  @Override
  public Layer writeMasterRecord(Layer record) throws InternalIOException {
    return rw.writeMasterRecord(record);
  }

  @Override
  public Layer writeMasterRecord(String id, Layer record) throws InternalIOException {
    return rw.writeMasterRecord(id, record);
  }

  @Override
  public List<Layer> readMasterRecords() throws InternalIOException {
    return rw.readMasterRecords();
  }

  @Override
  public void deleteMasterRecord(String id) throws InternalIOException {
    rw.deleteMasterRecord(id);
  }
  
}
